@@ -4,7 +4,7 @@ libclang is too huge for simple c++ analysis.
 import bisect
 import re
 from bisect import bisect_left, bisect_right
-from dataclasses import dataclass
+# from dataclasses import dataclass
 from enum import Enum
 from typing import Dict, List, Tuple
 
@@ -59,33 +59,34 @@ def find_list_str_prefix(data: List[str], prefix: str, full_match=False):
         yield i
 
 
-@dataclass
 class IdentifierMeta:
-    name: str
-    start: int
-    end: int
+    def __init__(self, name: str, start: int, end: int):
+        self.name = name 
+        self.start = start 
+        self.end = end 
 
-
-@dataclass
 class ClassDef:
-    name: str
-    keyword_pos: int
-    body_start: int
-    body_end: int
-    local_id: str = ""
-    is_template: bool = False
+    def __init__(self, name: str, keyword_pos: int, body_start: int,
+        body_end: int, local_id: str = "", is_template: bool = False):
+        self.name = name 
+        self.keyword_pos = keyword_pos 
+        self.body_start = body_start 
+        self.body_end = body_end 
+        self.local_id = local_id 
+        self.is_template = is_template 
 
-
-@dataclass
 class FunctionDef:
-    name: str
-    identifier_pos: int
-    param_start: int
-    param_end: int
-    body_start: int
-    body_end: int
-    is_template: bool
-    local_id: str = ""
+    def __init__(self, name: str, identifier_pos: int, param_start: int,
+        param_end: int, body_start: int,
+        body_end: int, local_id: str = "", is_template: bool = False):
+        self.name = name 
+        self.identifier_pos = identifier_pos 
+        self.param_start = param_start 
+        self.param_end = param_end 
+        self.body_start = body_start 
+        self.body_end = body_end 
+        self.local_id = local_id 
+        self.is_template = is_template 
 
 
 class CppSourceIterator(object):
@@ -151,7 +152,7 @@ class CppSourceIterator(object):
     def find_symbols_in_range(self, sym, start, end=None):
         if sym not in self._symbol_to_poses:
             raise KeyError(
-                f"unknown sym {sym}. available: {list(self._symbol_to_poses.keys())}"
+                "unknown sym {}. available: {}".format(sym, list(self._symbol_to_poses.keys()))
             )
         poses = self._symbol_to_poses[sym]
         hi = len(poses)
@@ -261,14 +262,14 @@ class CppSourceIterator(object):
                 end_bracket = end_brackets[val]
                 if not bracket_stack[end_bracket]:
                     raise ValueError(
-                        f"unbalanced bracket '{val}'({self.pos}) in your source."
+                        "unbalanced bracket '{}'({}) in your source.".format(val, self.pos)
                     )
                 start_val, start = bracket_stack[end_bracket].pop()
                 pairs.append((start_val, start, self.pos))
             self.pos += 1
             self.skip_string_comment()
         for k, v in bracket_stack.items():
-            assert len(v) == 0, f"unbalanced bracket {k} in your source."
+            assert len(v) == 0, "unbalanced bracket {} in your source.".format(k)
         return pairs, symbol_to_poses
 
     def get_namespace_ranges(self, class_defs: List[ClassDef]):
@@ -374,18 +375,35 @@ class CppSourceIterator(object):
                     if next_semi is not None:
                         func_meta = FunctionDef(meta.name, meta.start,
                                                 round_pair[0], round_pair[1],
-                                                -1, -1, is_template, func_id)
+                                                -1, -1, func_id, is_template)
                         res.append(func_meta)
 
                 continue
 
             func_meta = FunctionDef(meta.name, meta.start, round_pair[0],
                                     round_pair[1], curly_pair[0],
-                                    curly_pair[1], is_template, func_id)
+                                    curly_pair[1], func_id, is_template)
             res.append(func_meta)
         self.restore_state(state)
         return res
 
+    def find_marked_identifier(self, mark: str):
+        state = self.state()
+        res = []  # type: List[Tuple[str,str]]
+        for meta in self.find_identifier_prefix(mark, full_match=True):
+            self.reset_bracket_count().move(meta.end)
+            meta = self.next_identifier()
+            if meta is None:
+                continue
+            namespaces = []
+            for ns, ns_start, ns_end in self._namespace_ranges:
+                if meta.start > ns_start and meta.start < ns_end:
+                    namespaces.append(ns)
+
+            res.append((meta.name, "::".join(namespaces)))
+        self.restore_state(state)
+        return res
+
     def inc_bracket(self):
         val = self.source[self.pos]
         if val not in self._bracket_inc:
@@ -436,6 +454,7 @@ class CppSourceIterator(object):
                 continue
             if self.pos in self.identifier_start_to_meta:
                 meta = self.identifier_start_to_meta[self.pos]
+                self.pos = meta.end
                 return meta
             else:
                 return None
@@ -472,3 +491,26 @@ class CppSourceIterator(object):
                 yield meta
             else:
                 break
+
+
+if __name__ == "__main__":
+    source = """
+    std::cout << L"hello" << " world";
+    std::cout << "He said: \"bananas\"" << "...";
+    std::cout << ("");
+    std::cout << "\x12\23\x34";
+    "" // empty string
+    '"' // character literal
+
+    // this is "a string literal" in a comment
+    /* this is
+    "also inside"
+    //a comment */
+
+    // and this /*
+    "is not in a comment"
+    // */
+
+    """
+    it = CppSourceIterator(source)
+    print(it.bracket_pairs)
\ No newline at end of file
